﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Imaging;
using System.Runtime.InteropServices;

using Microsoft.Research.Kinect.Nui;
using Microsoft.Research.Kinect.Audio;

namespace depthID
{
    public partial class Form1 : Form
    {
        Runtime nui;

        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            nui = Runtime.Kinects[0];
            nui.Initialize(RuntimeOptions.UseDepthAndPlayerIndex | RuntimeOptions.UseSkeletalTracking);
            nui.DepthStream.Open(ImageStreamType.Depth, 2, ImageResolution.Resolution320x240, ImageType.DepthAndPlayerIndex);
            nui.DepthFrameReady += new EventHandler<ImageFrameReadyEventArgs>(nui_DepthFrameReady);
        }

        void nui_DepthFrameReady(object sender, ImageFrameReadyEventArgs e)
        {
            PlanarImage Image = e.ImageFrame.Image;
            int[] depth = new int[Image.Width * Image.Height];
            int[] player = new int[Image.Width * Image.Height];
            int[] playercoded = new int[Image.Width * Image.Height];

            for (int i = 0; i < depth.Length; i++)
            {
                depth[i] = (Image.Bits[i * 2 + 1] << 5) | (Image.Bits[i * 2] >> 3);
                player[i] = Image.Bits[i * 2] & 0x07;
                playercoded[i] = 0;
                if ((player[i] & 0x01) == 0x01) playercoded[i] |= 0xFF0000;
                if ((player[i] & 0x02) == 0x02) playercoded[i] |= 0x00FF00;
                if ((player[i] & 0x04) == 0x04) playercoded[i] |= 0x0000FF;
            }

            pictureBox1.Image = IntToBitmap(depth, Image.Width, Image.Height);
            pictureBox2.Image = IntToBitmap(playercoded, Image.Width, Image.Height);
        }


        Bitmap IntToBitmap(int[] array, int w, int h)
        {
            Bitmap bmap = new Bitmap(
              w,
              h,
              PixelFormat.Format32bppRgb);
            BitmapData bmapdata = bmap.LockBits(
              new Rectangle(0, 0, w, h),
              ImageLockMode.WriteOnly,
              bmap.PixelFormat);
              IntPtr ptr = bmapdata.Scan0;
              Marshal.Copy(array,
              0,
              ptr,
              array.Length);
            bmap.UnlockBits(bmapdata);
            return bmap;
        }

        Bitmap DepthToBitmap(PlanarImage PImage)
        {
            Bitmap bmap = new Bitmap(
             PImage.Width,
             PImage.Height,
             PixelFormat.Format16bppRgb555);
             BitmapData bmapdata = bmap.LockBits(
             new Rectangle(0, 0, PImage.Width,
             PImage.Height),
             ImageLockMode.WriteOnly,
             bmap.PixelFormat);
            IntPtr ptr = bmapdata.Scan0;
            Marshal.Copy(PImage.Bits,
             0,
             ptr,
             PImage.Width *
             PImage.BytesPerPixel *
             PImage.Height);
            bmap.UnlockBits(bmapdata);
            return bmap;
        }
   

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            nui.Uninitialize();
            Environment.Exit(0);
        }
    }
}
